import { BrowserModule } from '@angular/platform-browser';
import { Component, Directive, ElementRef, EventEmitter, HostListener, Injectable, Input, NgModule, Output, Pipe, forwardRef } from '@angular/core';
import { CommonModule, DatePipe } from '@angular/common';
import { FormsModule, NG_VALIDATORS, NG_VALUE_ACCESSOR } from '@angular/forms';

var HacDropdownFilterPipe = (function () {
    function HacDropdownFilterPipe() {
    }
    /**
     * @param {?} value
     * @param {?} filter
     * @param {?=} apply
     * @return {?}
     */
    HacDropdownFilterPipe.prototype.transform = function (value, filter, apply) {
        if (apply === void 0) { apply = true; }
        if (!filter || !apply)
            return value;
        return value.filter(function (o) { return o.label.toLowerCase().indexOf(filter.toLowerCase()) > -1 || o.key.toString().toLowerCase() === filter.toLowerCase(); });
    };
    HacDropdownFilterPipe.decorators = [
        { type: Injectable },
        { type: Pipe, args: [{
                    name: 'hacDropdownFilter'
                },] },
    ];
    /**
     * @nocollapse
     */
    HacDropdownFilterPipe.ctorParameters = function () { return []; };
    return HacDropdownFilterPipe;
}());

var HacDropdownComponent = (function () {
    /**
     * @param {?} elementRef
     * @param {?} dropdownFilter
     */
    function HacDropdownComponent(elementRef, dropdownFilter) {
        this.elementRef = elementRef;
        this.dropdownFilter = dropdownFilter;
        this.optionGroups = [];
        this.placeholder = 'Select';
        this.allowEmpty = false;
        this.filtrable = false;
        this.selectedChange = new EventEmitter();
        this.disabled = false;
        this.collapsed = true;
        this.subscriptions = [];
        this.windowHeight = 0;
        this.onTouchedCallback = function () { };
        this.onChangeCallback = function () { };
        this.syncWindowHeight();
    }
    Object.defineProperty(HacDropdownComponent.prototype, "selected", {
        /**
         * @return {?}
         */
        get: function () {
            return this._selected;
        },
        /**
         * @param {?} v
         * @return {?}
         */
        set: function (v) {
            var _this = this;
            this._selected = v;
            this.onChangeCallback(this._selected);
            // ensure valid values, if not, reset
            if (v && !this.findOptionByKey(v)) {
                setTimeout(function () {
                    _this._selected = null;
                    _this.selectedChange.emit(_this._selected);
                    _this.onChangeCallback(_this._selected);
                }, 0);
            }
            this.updateFilter();
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @return {?}
     */
    HacDropdownComponent.prototype.ngOnDestroy = function () {
        this.subscriptions.forEach(function (s) { return s.unsubscribe(); });
    };
    /**
     * @param {?} event
     * @return {?}
     */
    HacDropdownComponent.prototype.onClick = function (event) {
        var /** @type {?} */ clickOutsideComponent = !this.elementRef.nativeElement.contains(event.target);
        var /** @type {?} */ srcHtmlFor = ((event.srcElement)).htmlFor;
        var /** @type {?} */ clickFromLabelFor = (this.id && srcHtmlFor && srcHtmlFor === this.id);
        if (clickOutsideComponent && !clickFromLabelFor && !this.collapsed) {
            this.closeDropdown();
        }
    };
    /**
     * @param {?} event
     * @return {?}
     */
    HacDropdownComponent.prototype.onResize = function (event) {
        this.syncWindowHeight();
    };
    /**
     * @param {?} event
     * @return {?}
     */
    HacDropdownComponent.prototype.onScroll = function (event) {
        this.syncWindowHeight();
    };
    /**
     * @return {?}
     */
    HacDropdownComponent.prototype.getSelected = function () {
        if (this.hasGroups()) {
            return this.findOptionByKey(this._selected);
        }
        return null;
    };
    /**
     * @param {?} key
     * @return {?}
     */
    HacDropdownComponent.prototype.select = function (key) {
        this.selected = key;
        this.closeDropdown();
        this.selectedChange.emit(this.selected);
    };
    /**
     * @return {?}
     */
    HacDropdownComponent.prototype.hasGroups = function () {
        return this.optionGroups && this.optionGroups.length > 0;
    };
    /**
     * @param {?=} e
     * @return {?}
     */
    HacDropdownComponent.prototype.openDropdown = function (e) {
        if (this.disabled)
            return;
        this.collapsed = false;
        if (e) {
            e.target.focus();
        }
    };
    /**
     * @param {?=} e
     * @return {?}
     */
    HacDropdownComponent.prototype.closeDropdown = function (e) {
        this.collapsed = true;
        this.onTouchedCallback();
    };
    /**
     * @param {?=} e
     * @return {?}
     */
    HacDropdownComponent.prototype.toggleDropdown = function (e) {
        if (this.collapsed) {
            this.openDropdown();
        }
        else {
            this.closeDropdown();
        }
        if (e) {
            e.stopPropagation();
        }
    };
    /**
     * @return {?}
     */
    HacDropdownComponent.prototype.shouldApplyFilter = function () {
        var /** @type {?} */ selected = this.getSelected();
        return selected && this.filter !== selected.label;
    };
    /**
     * @param {?} e
     * @return {?}
     */
    HacDropdownComponent.prototype.handleKey = function (e) {
        switch (e.keyCode) {
            case 13:// Enter key
                this.selectFirstMatchingOption();
                break;
            case 27://ESC key
                this.blurFilter();
                this.closeDropdown();
                break;
        }
    };
    /**
     * @param {?} obj
     * @return {?}
     */
    HacDropdownComponent.prototype.writeValue = function (obj) {
        this.selected = obj;
    };
    /**
     * @param {?} fn
     * @return {?}
     */
    HacDropdownComponent.prototype.registerOnChange = function (fn) {
        this.onChangeCallback = fn;
    };
    /**
     * @param {?} fn
     * @return {?}
     */
    HacDropdownComponent.prototype.registerOnTouched = function (fn) {
        this.onTouchedCallback = fn;
    };
    /**
     * @param {?} isDisabled
     * @return {?}
     */
    HacDropdownComponent.prototype.setDisabledState = function (isDisabled) {
        this.disabled = isDisabled;
    };
    /**
     * @return {?}
     */
    HacDropdownComponent.prototype.getFilterElem = function () {
        return this.elementRef.nativeElement.querySelector('.js-filter');
    };
    /**
     * @return {?}
     */
    HacDropdownComponent.prototype.calcDropdownHeight = function () {
        var /** @type {?} */ el = this.elementRef.nativeElement.querySelector('.dropdown-menu');
        var /** @type {?} */ pos = this.getPos(el);
        var /** @type {?} */ height = window.innerHeight - 10 - pos.y;
        // height = window.innerHeight - (window.pageYOffset || document.documentElement.scrollTop) - (el.clientTop || 0);
        if (!height || height <= 150) {
            height = 150;
        }
        return height + "px";
    };
    /**
     * @param {?} key
     * @return {?}
     */
    HacDropdownComponent.prototype.findOptionByKey = function (key) {
        return this.optionGroups.map(function (f) { return f.options.find(function (o) { return o.key === key; }); }).find(function (o) { return o != null; });
    };
    /**
     * @return {?}
     */
    HacDropdownComponent.prototype.updateFilter = function () {
        var /** @type {?} */ selection = this.getSelected();
        this.filter = selection ? selection.label : null;
    };
    /**
     * @return {?}
     */
    HacDropdownComponent.prototype.syncWindowHeight = function () {
        this.windowHeight = window.innerHeight;
    };
    /**
     * @param {?} el
     * @return {?}
     */
    HacDropdownComponent.prototype.getPos = function (el) {
        var /** @type {?} */ lx, /** @type {?} */ ly;
        for (lx = 0, ly = 0; el != null; lx += el.offsetLeft, ly += el.offsetTop, el = el.offsetParent) { }
        
        return { x: lx, y: ly };
    };
    /**
     * @return {?}
     */
    HacDropdownComponent.prototype.blurFilter = function () {
        this.elementRef.nativeElement.querySelector('input').blur();
    };
    /**
     * @return {?}
     */
    HacDropdownComponent.prototype.selectFirstMatchingOption = function () {
        for (var _i = 0, _a = this.optionGroups; _i < _a.length; _i++) {
            var group = _a[_i];
            var /** @type {?} */ candidates = this.dropdownFilter.transform(group.options, this.filter);
            if (candidates.length > 0) {
                this.select(candidates[0].key);
                this.blurFilter();
                break;
            }
        }
    };
    /* Dropdown styling dimensions */
    HacDropdownComponent.decorators = [
        { type: Component, args: [{
                    selector: 'hac-dropdown',
                    template: "<div class=\"hac-dropdown dropdown\" [ngClass]=\"{ 'open show': !collapsed }\" > <input type=\"text\" class=\"form-control js-filter\" [placeholder]=\"placeholder\" id=\"{{id}}\" [(ngModel)]=\"filter\" [readonly]=\"!filtrable || disabled\" (keyup)=\"handleKey($event)\" (focus)=\"openDropdown($event)\" autocomplete=\"off\" /> <i class=\"fa fa-angle-down fa-2x\" aria-hidden=\"true\" (click)=\"toggleDropdown($event)\"></i> <ul class=\"dropdown-menu\" role=\"menu\" *ngIf=\"hasGroups()\" [hacCopyWidth]=\"getFilterElem()\" widthCssProperty=\"width\" [skip]=\"collapsed\" [style.maxHeight]=\"calcDropdownHeight()\"> <li class=\"dropdown-item\"><a *ngIf=\"allowEmpty\" (click)=\"select(null)\">{{placeholder}}</a></li> <ng-container *ngFor=\"let group of optionGroups\"> <li *ngIf=\"group.label\" class=\"dropdown-item hac-dropdown-grouplabel\"><a><strong>{{group.label}}</strong></a></li> <li *ngFor=\"let option of group.options | hacDropdownFilter: filter : shouldApplyFilter()\" (click)=\"select(option.key)\" class=\"dropdown-item\" [ngClass]=\"{ 'active': option.key === selected, 'hac-dropdown-groupitem': group.label != null }\"> <a>{{option.label}}</a> </li> </ng-container> </ul> </div>",
                    providers: [
                        HacDropdownFilterPipe,
                        {
                            provide: NG_VALUE_ACCESSOR,
                            useExisting: forwardRef(function () { return HacDropdownComponent; }),
                            multi: true
                        }
                    ]
                },] },
    ];
    /**
     * @nocollapse
     */
    HacDropdownComponent.ctorParameters = function () { return [
        { type: ElementRef, },
        { type: HacDropdownFilterPipe, },
    ]; };
    HacDropdownComponent.propDecorators = {
        'optionGroups': [{ type: Input },],
        'placeholder': [{ type: Input },],
        'allowEmpty': [{ type: Input },],
        'filtrable': [{ type: Input },],
        'selectedChange': [{ type: Output },],
        'id': [{ type: Input },],
        'disabled': [{ type: Input },],
        'selected': [{ type: Input },],
        'onClick': [{ type: HostListener, args: ['document:click', ['$event'],] },],
        'onResize': [{ type: HostListener, args: ['window:resize', ['$event'],] },],
        'onScroll': [{ type: HostListener, args: ['window:scroll', ['$event'],] },],
    };
    return HacDropdownComponent;
}());

var DateHelper = (function () {
    function DateHelper() {
    }
    /**
     * Format date to 'yyyy-MM-dd' format
    \@param day
     * @param {?} day
     * @return {?}
     */
    DateHelper.formatIsoDate = function (day) {
        var /** @type {?} */ d = this.ensureDateObject(day);
        var /** @type {?} */ monthNum = d.getMonth() + 1;
        return d.getFullYear() + "-" + (monthNum < 10 ? '0' : '') + monthNum + "-" + (d.getDate() < 10 ? '0' : '') + d.getDate();
    };
    /**
     * Verify if both dates are the same day
    \@param day1
    \@param day2
     * @param {?} day1
     * @param {?} day2
     * @return {?}
     */
    DateHelper.areDatesEqual = function (day1, day2) {
        return day1 && day2 && this.formatIsoDate(day1) === this.formatIsoDate(day2);
    };
    /**
     * Checks if candidate is in range between startRange and endRange including both limits
    \@param candidate
    \@param startRange
    \@param endRange
     * @param {?} candidate
     * @param {?} startRange
     * @param {?} endRange
     * @return {?}
     */
    DateHelper.isInRange = function (candidate, startRange, endRange) {
        if (!candidate || !startRange || !endRange)
            return false;
        var /** @type {?} */ candidateFormatted = this.formatIsoDate(candidate);
        return candidateFormatted >= this.formatIsoDate(startRange) && candidateFormatted <= this.formatIsoDate(endRange);
    };
    /**
     * Checks if day1 is greater than day 2
    \@param day1
    \@param day2
     * @param {?} day1
     * @param {?} day2
     * @return {?}
     */
    DateHelper.isGreater = function (day1, day2) {
        return day1 && day2 && this.formatIsoDate(day1) > this.formatIsoDate(day2);
    };
    /**
     * Checks if day1 is greater or equal than day 2
    \@param day1
    \@param day2
     * @param {?} day1
     * @param {?} day2
     * @return {?}
     */
    DateHelper.isGreaterOrEqual = function (day1, day2) {
        return day1 && day2 && this.formatIsoDate(day1) >= this.formatIsoDate(day2);
    };
    /**
     * Get today date at 00:00h
     * @return {?}
     */
    DateHelper.today = function () {
        var /** @type {?} */ today = new Date();
        this.resetTime(today);
        return today;
    };
    /**
     * Resets time part from Date object to 00:00:00.000h
     * @param {?} date
     * @return {?}
     */
    DateHelper.resetTime = function (date) {
        if (typeof date === 'string') {
            date = this.formatIsoDate(date);
            return;
        }
        date.setHours(0);
        date.setMinutes(0);
        date.setSeconds(0);
        date.setMilliseconds(0);
    };
    /**
     * @param {?} date
     * @return {?}
     */
    DateHelper.ensureDateObject = function (date) {
        return typeof date === 'string' ? new Date(date) : date;
    };
    return DateHelper;
}());

/*
 <  January 2017   >
_____________________
[M][T][W][T][F][S][S]
---------------------
 _  1  2  3  4  5  6
 7  8  9  10 11 12 13
 14 15 16 17 18 19 20
 21 22 23 24 25 26 27
 28 29 30 31 __ __ __

*/
var WeekDay = {};
WeekDay.Sunday = 0;
WeekDay.Monday = 1;
WeekDay.Tuesday = 2;
WeekDay.Wednesday = 3;
WeekDay.Thursday = 4;
WeekDay.Friday = 5;
WeekDay.Saturday = 6;
WeekDay[WeekDay.Sunday] = "Sunday";
WeekDay[WeekDay.Monday] = "Monday";
WeekDay[WeekDay.Tuesday] = "Tuesday";
WeekDay[WeekDay.Wednesday] = "Wednesday";
WeekDay[WeekDay.Thursday] = "Thursday";
WeekDay[WeekDay.Friday] = "Friday";
WeekDay[WeekDay.Saturday] = "Saturday";
var weekDayList = [
    WeekDay.Monday,
    WeekDay.Tuesday,
    WeekDay.Wednesday,
    WeekDay.Thursday,
    WeekDay.Friday,
    WeekDay.Saturday,
    WeekDay.Sunday
];
var startWeekDay = WeekDay.Monday;
var HacCalendarDayModel = (function () {
    /**
     * @param {?} day
     */
    function HacCalendarDayModel(day) {
        this.day = DateHelper.ensureDateObject(day);
    }
    /**
     * @return {?}
     */
    HacCalendarDayModel.prototype.isToday = function () {
        return DateHelper.areDatesEqual(this.day, new Date());
    };
    return HacCalendarDayModel;
}());
var HacCalendarWeekModel = (function () {
    function HacCalendarWeekModel() {
        this.days = [];
    }
    /**
     * @param {?} day
     * @return {?}
     */
    HacCalendarWeekModel.prototype.addDay = function (day) {
        if (this.days.length === 0 && day.getDay() !== startWeekDay) {
            var /** @type {?} */ previousDay = new Date(day);
            previousDay.setDate(previousDay.getDate() - 1);
            this.addDay(previousDay);
        }
        this.days.push(new HacCalendarDayModel(day));
    };
    return HacCalendarWeekModel;
}());
var HacCalendarModel = (function () {
    /**
     * @param {?} month
     */
    function HacCalendarModel(month) {
        this.weeks = [];
        this.month = new Date(month.getFullYear(), month.getMonth(), 1, 0, 0, 0, 0);
        this.buildCalendar();
    }
    /**
     * @return {?}
     */
    HacCalendarModel.prototype.buildCalendar = function () {
        var /** @type {?} */ day = new Date(this.month);
        var /** @type {?} */ lastDay = this.getMonthLastDay();
        while (day <= lastDay) {
            // add new week when on first week day or first week
            if (day.getDay() === startWeekDay || this.weeks.length === 0) {
                var /** @type {?} */ week = new HacCalendarWeekModel();
                week.addDay(new Date(day));
                this.weeks.push(week);
            }
            else {
                this.weeks[this.weeks.length - 1].addDay(new Date(day));
            }
            day.setDate(day.getDate() + 1); // next day
        }
        // add remaining days of last week from next month
        var /** @type {?} */ lastWeek = this.weeks[this.weeks.length - 1];
        while (lastWeek.days.length < 7) {
            lastWeek.addDay(new Date(day));
            day.setDate(day.getDate() + 1);
        }
    };
    /**
     * @param {?} day
     * @return {?}
     */
    HacCalendarModel.prototype.contains = function (day) {
        return day.day.getFullYear() === this.month.getFullYear() && day.day.getMonth() === this.month.getMonth();
    };
    /**
     * @return {?}
     */
    HacCalendarModel.prototype.getMonthLastDay = function () {
        return new Date(this.month.getFullYear(), this.month.getMonth() + 1, 0);
    };
    /**
     * @return {?}
     */
    HacCalendarModel.prototype.getPreviousMonthLastDay = function () {
        return new Date(this.month.getFullYear(), this.month.getMonth(), 0);
    };
    return HacCalendarModel;
}());

var HacWeekDayFormatter = (function () {
    /**
     * @param {?} datePipe
     */
    function HacWeekDayFormatter(datePipe) {
        this.datePipe = datePipe;
        this.Monday = new Date(2016, 1, 1, 0, 0, 0, 0);
        this.Tuesday = new Date(2016, 1, 2, 0, 0, 0, 0);
        this.Wednesday = new Date(2016, 1, 3, 0, 0, 0, 0);
        this.Thursday = new Date(2016, 1, 4, 0, 0, 0, 0);
        this.Friday = new Date(2016, 1, 5, 0, 0, 0, 0);
        this.Saturday = new Date(2016, 1, 6, 0, 0, 0, 0);
        this.Sunday = new Date(2016, 1, 7, 0, 0, 0, 0);
    }
    /**
     * @param {?} value
     * @param {?=} customFormat
     * @return {?}
     */
    HacWeekDayFormatter.prototype.transform = function (value, customFormat) {
        if (customFormat === void 0) { customFormat = 'EEE'; }
        if (value != null) {
            switch (value) {
                case WeekDay.Monday:
                    return this.datePipe.transform(this.Monday, customFormat);
                case WeekDay.Tuesday:
                    return this.datePipe.transform(this.Tuesday, customFormat);
                case WeekDay.Wednesday:
                    return this.datePipe.transform(this.Wednesday, customFormat);
                case WeekDay.Thursday:
                    return this.datePipe.transform(this.Thursday, customFormat);
                case WeekDay.Friday:
                    return this.datePipe.transform(this.Friday, customFormat);
                case WeekDay.Saturday:
                    return this.datePipe.transform(this.Saturday, customFormat);
                case WeekDay.Sunday:
                    return this.datePipe.transform(this.Sunday, customFormat);
            }
        }
        return '';
    };
    HacWeekDayFormatter.decorators = [
        { type: Injectable },
        { type: Pipe, args: [{
                    name: 'hacWeekDayFormatter'
                },] },
    ];
    /**
     * @nocollapse
     */
    HacWeekDayFormatter.ctorParameters = function () { return [
        { type: DatePipe, },
    ]; };
    return HacWeekDayFormatter;
}());

var HacImmutableHelper = (function () {
    function HacImmutableHelper() {
    }
    /**
     * @template T
     * @param {?} obj
     * @param {?} updatedProperties
     * @return {?}
     */
    HacImmutableHelper.set = function (obj, updatedProperties) {
        return Object.assign({}, obj, updatedProperties);
    };
    return HacImmutableHelper;
}());

var HacDatepickerComponent = (function () {
    /**
     * @param {?} elementRef
     */
    function HacDatepickerComponent(elementRef) {
        this.elementRef = elementRef;
        this.startDateChange = new EventEmitter();
        this.endDateChange = new EventEmitter();
        this._options = {};
        this.disabled = false;
        this.calendars = [];
        this.weekDayList = weekDayList;
        this._collapsed = true;
        this.onTouchedCallback = function () { };
        this.onChangeCallback = function () { };
        this.onValidatorChangeCallback = function () { };
    }
    Object.defineProperty(HacDatepickerComponent.prototype, "startDate", {
        /**
         * @return {?}
         */
        get: function () {
            return this._startDate;
        },
        /**
         * @param {?} v
         * @return {?}
         */
        set: function (v) {
            var _this = this;
            if (DateHelper.areDatesEqual(v, this._startDate))
                return;
            this._startDate = v;
            this.pushNewModelChange();
            if (!this._startDate)
                return;
            // reset invalid dates (disabled day or past time)
            if (this.isDisabled(new HacCalendarDayModel(v), 'start')) {
                setTimeout(function () {
                    _this.setStartDate(null);
                    _this.forceSelectionKind('start');
                }, 0);
            }
            else if (this.options.range && this._endDate && DateHelper.isGreater(v, this._endDate)) {
                setTimeout(function () {
                    _this.setEndDate(null);
                    _this.forceSelectionKind('end');
                }, 0);
            }
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(HacDatepickerComponent.prototype, "endDate", {
        /**
         * @return {?}
         */
        get: function () {
            return this._endDate;
        },
        /**
         * @param {?} v
         * @return {?}
         */
        set: function (v) {
            var _this = this;
            if (DateHelper.areDatesEqual(v, this._endDate))
                return;
            this._endDate = v;
            this.pushNewModelChange();
            if (!this._endDate)
                return;
            // reset invalid dates (disabled day or past time)
            if (this.isDisabled(new HacCalendarDayModel(v), 'end')
                || (this.options.range && this._startDate && DateHelper.isGreater(this._startDate, v))) {
                setTimeout(function () {
                    _this.setEndDate(null);
                    _this.forceSelectionKind('end');
                }, 0);
            }
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(HacDatepickerComponent.prototype, "selected", {
        /**
         * @return {?}
         */
        get: function () {
            return this.startDate;
        },
        /**
         * @param {?} v
         * @return {?}
         */
        set: function (v) {
            this.setStartDate(v);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(HacDatepickerComponent.prototype, "selectedChange", {
        /**
         * @return {?}
         */
        get: function () {
            return this.startDateChange;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(HacDatepickerComponent.prototype, "options", {
        /**
         * @return {?}
         */
        get: function () {
            return this._options;
        },
        /**
         * @param {?} v
         * @return {?}
         */
        set: function (v) {
            this._options = v;
            this.setOptionsDefaults();
            this.buildCalendarModel();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(HacDatepickerComponent.prototype, "collapsed", {
        /**
         * @return {?}
         */
        get: function () {
            return this._collapsed;
        },
        /**
         * @param {?} v
         * @return {?}
         */
        set: function (v) {
            if (this.disabled) {
                this._collapsed = true;
                this.emitFocusEvent();
                return;
            }
            // mark as touched when closing
            if (!this._collapsed && v && this.onTouchedCallback) {
                this.onTouchedCallback();
            }
            this.animate();
            this._collapsed = v;
            this.emitFocusEvent();
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @return {?}
     */
    HacDatepickerComponent.prototype.ngOnInit = function () {
        this.setOptionsDefaults();
        this.buildCalendarModel();
    };
    /**
     * @param {?} event
     * @return {?}
     */
    HacDatepickerComponent.prototype.onClick = function (event) {
        if (this.options.elementId && event.target.htmlFor === this.options.elementId) {
            this.collapsed = false;
        }
        else if (!this.elementRef.nativeElement.contains(event.target)) {
            if (!this.collapsed) {
                this.collapsed = true;
                if (this.onTouchedCallback)
                    this.onTouchedCallback();
            }
        }
    };
    /**
     * @return {?}
     */
    HacDatepickerComponent.prototype.buildCalendarModel = function () {
        this.calendars = [];
        var /** @type {?} */ currentMonth = new Date(this.options.currentDisplayMonth);
        var /** @type {?} */ calendarMonths = this.options.showMonths > 0 ? this.options.showMonths : 1;
        for (var /** @type {?} */ i = 0; i < calendarMonths; i++) {
            this.calendars.push(new HacCalendarModel(currentMonth));
            currentMonth.setMonth(currentMonth.getMonth() + 1);
        }
    };
    /**
     * @param {?} day
     * @return {?}
     */
    HacDatepickerComponent.prototype.select = function (day) {
        if (this.isDisabled(day)) {
            return;
        }
        if (!this.options.range || this.getSelectionKind() === 'start') {
            this.setStartDate(day.day);
            if (!this.forcedSelectionKind) {
                this.setEndDate(null);
            }
            else if (this.options.range) {
                this.forceSelectionKind('end');
                if (day.day > this._endDate) {
                    this.setEndDate(null);
                }
            }
            this.collapsed = !this.options.range;
        }
        else {
            this.setEndDate(day.day);
            this.collapsed = true;
            this.forceSelectionKind(null);
        }
    };
    /**
     * @param {?} day
     * @return {?}
     */
    HacDatepickerComponent.prototype.isSelected = function (day) {
        return DateHelper.areDatesEqual(day.day, this._startDate)
            || this.options.range && DateHelper.areDatesEqual(day.day, this._endDate);
    };
    /**
     * @param {?} day
     * @param {?=} selectionKind
     * @return {?}
     */
    HacDatepickerComponent.prototype.isDisabled = function (day, selectionKind) {
        selectionKind = selectionKind || this.getSelectionKind();
        var /** @type {?} */ exceedsMinDate = this.options.minDate && DateHelper.isGreater(this.options.minDate, day.day);
        if (exceedsMinDate)
            return true;
        var /** @type {?} */ exceedsMaxDate = this.options.maxDate && DateHelper.isGreater(day.day, this.options.maxDate);
        if (exceedsMaxDate)
            return true;
        var /** @type {?} */ isBlacklisted = this.options.dayListKind === 'blacklist' && this.isInDayList(day, selectionKind);
        isBlacklisted = isBlacklisted || this.options.dayListKind === 'whitelist' && !this.isInDayList(day, selectionKind);
        if (isBlacklisted)
            return true;
        var /** @type {?} */ isPastOverflow = selectionKind === 'end' && DateHelper.isGreater(this._startDate, day.day);
        return isPastOverflow;
    };
    /**
     * @param {?} day
     * @return {?}
     */
    HacDatepickerComponent.prototype.isStart = function (day) {
        return this.options.range && DateHelper.areDatesEqual(day.day, this._startDate);
    };
    /**
     * @param {?} day
     * @return {?}
     */
    HacDatepickerComponent.prototype.isEnd = function (day) {
        var /** @type {?} */ lastDayRange = this.getLastRangeDay();
        return this.options.range && DateHelper.areDatesEqual(day.day, lastDayRange);
    };
    /**
     * @param {?} day
     * @param {?=} useHover
     * @return {?}
     */
    HacDatepickerComponent.prototype.inRange = function (day, useHover) {
        if (useHover === void 0) { useHover = false; }
        if (!this.options.range)
            return false;
        var /** @type {?} */ startDate = this._startDate;
        var /** @type {?} */ endDate = this.getLastRangeDay();
        if (!startDate || !endDate)
            return false;
        return this.options.range && DateHelper.isInRange(day.day, startDate, endDate);
    };
    /**
     * @param {?} day
     * @return {?}
     */
    HacDatepickerComponent.prototype.setHoverDate = function (day) {
        this.hoverDate = day ? day.day : null;
    };
    /**
     * @return {?}
     */
    HacDatepickerComponent.prototype.clearDates = function () {
        this.setStartDate(null);
        this.setEndDate(null);
    };
    /**
     * @param {?=} $event
     * @return {?}
     */
    HacDatepickerComponent.prototype.prevMonth = function ($event) {
        this.options.currentDisplayMonth.setMonth(this.options.currentDisplayMonth.getMonth() - 1);
        this.options = HacImmutableHelper.set(this.options, {
            currentDisplayMonth: this.options.currentDisplayMonth
        });
        this.buildCalendarModel();
        this.animate();
        $event.stopPropagation();
    };
    /**
     * @param {?=} $event
     * @return {?}
     */
    HacDatepickerComponent.prototype.nextMonth = function ($event) {
        this.options.currentDisplayMonth.setMonth(this.options.currentDisplayMonth.getMonth() + 1);
        this.options = HacImmutableHelper.set(this.options, {
            currentDisplayMonth: this.options.currentDisplayMonth
        });
        this.buildCalendarModel();
        this.animate();
        $event.stopPropagation();
    };
    /**
     * @param {?} event
     * @return {?}
     */
    HacDatepickerComponent.prototype.toggleCalendar = function (event) {
        this.collapsed = !this.collapsed;
    };
    /**
     * @return {?}
     */
    HacDatepickerComponent.prototype.getSelectionKind = function () {
        if (this.forcedSelectionKind) {
            return this.forcedSelectionKind;
        }
        return !this._startDate || (this.options.range && this._startDate && this._endDate) ? 'start' : 'end';
    };
    /**
     * @param {?=} kind
     * @return {?}
     */
    HacDatepickerComponent.prototype.forceSelectionKind = function (kind) {
        this.forcedSelectionKind = kind;
        if (kind) {
            this.collapsed = false;
        }
    };
    /**
     * @return {?}
     */
    HacDatepickerComponent.prototype.selectToday = function () {
        var /** @type {?} */ today = new HacCalendarDayModel(DateHelper.today());
        this.select(today);
    };
    /**
     * @return {?}
     */
    HacDatepickerComponent.prototype.isTodayActionEnabled = function () {
        return this._options.enableTodayAction && !this.isDisabled(new HacCalendarDayModel(DateHelper.today()));
    };
    /**
     * @return {?}
     */
    HacDatepickerComponent.prototype.isClearActionEnabled = function () {
        return this._options.enableClearAction;
    };
    /**
     * @return {?}
     */
    HacDatepickerComponent.prototype.getSelectorElem = function () {
        return this.elementRef.nativeElement.querySelector('.hac-cal-selectorwrapper');
    };
    /**
     * @param {?} calendarIndex
     * @return {?}
     */
    HacDatepickerComponent.prototype.isPrevArrowVisible = function (calendarIndex) {
        var /** @type {?} */ isInRange = !this.getMinDate() || (this.getMinDate() &&
            DateHelper.isGreaterOrEqual(this.calendars[calendarIndex].getPreviousMonthLastDay(), this.getMinDate()));
        return calendarIndex === 0 && isInRange;
    };
    /**
     * @param {?} calendarIndex
     * @return {?}
     */
    HacDatepickerComponent.prototype.isNextArrowVisible = function (calendarIndex) {
        var /** @type {?} */ isInRange = !this.getMaxDate() || (this.getMaxDate() &&
            DateHelper.isGreaterOrEqual(this.getMaxDate(), this.calendars[calendarIndex].getMonthLastDay()));
        return (calendarIndex === this.calendars.length - 1) && isInRange;
    };
    /**
     * @return {?}
     */
    HacDatepickerComponent.prototype.focus = function () {
        this.forceSelectionKind('start');
    };
    /**
     * @return {?}
     */
    HacDatepickerComponent.prototype.focusout = function () {
        if (this.onTouchedCallback)
            this.onTouchedCallback();
    };
    /**
     * @param {?} obj
     * @return {?}
     */
    HacDatepickerComponent.prototype.writeValue = function (obj) {
        if (this._options.range) {
            this.setStartDate(obj ? obj.startDate : null);
            this.setEndDate(obj ? obj.endDate : null);
        }
        else {
            this.setStartDate(obj);
        }
    };
    /**
     * @param {?} fn
     * @return {?}
     */
    HacDatepickerComponent.prototype.registerOnChange = function (fn) {
        this.onChangeCallback = fn;
    };
    /**
     * @param {?} fn
     * @return {?}
     */
    HacDatepickerComponent.prototype.registerOnTouched = function (fn) {
        this.onTouchedCallback = fn;
    };
    /**
     * @param {?} isDisabled
     * @return {?}
     */
    HacDatepickerComponent.prototype.setDisabledState = function (isDisabled) {
        this.disabled = isDisabled;
    };
    /**
     * @param {?} c
     * @return {?}
     */
    HacDatepickerComponent.prototype.validate = function (c) {
        var /** @type {?} */ errors = null;
        if (this.options.range) {
            if (this._startDate == null) {
                errors = errors || {};
                errors.startDateRequired = true;
            }
            if (this._endDate == null) {
                errors = errors || {};
                errors.endDateRequired = true;
            }
        }
        return errors;
    };
    /**
     * @param {?} fn
     * @return {?}
     */
    HacDatepickerComponent.prototype.registerOnValidatorChange = function (fn) {
        this.onValidatorChangeCallback = fn;
    };
    /**
     * @return {?}
     */
    HacDatepickerComponent.prototype.pushNewModelChange = function () {
        if (this.onValidatorChangeCallback) {
            this.onValidatorChangeCallback();
        }
        if (!this.onChangeCallback)
            return;
        var /** @type {?} */ model = null;
        if (!this._options.range) {
            model = this._startDate;
        }
        else if (this._startDate && this._endDate) {
            model = {
                startDate: this._startDate,
                endDate: this._endDate
            };
        }
        this.onChangeCallback(model);
    };
    /**
     * @return {?}
     */
    HacDatepickerComponent.prototype.setOptionsDefaults = function () {
        this._options = this._options || {};
        var /** @type {?} */ startDate = this._options.currentDisplayMonth ?
            new Date(this.options.currentDisplayMonth) :
            this._startDate ? DateHelper.ensureDateObject(this._startDate) : new Date();
        if ((this._options.minDate && DateHelper.isGreater(this._options.minDate, startDate)) ||
            (this._options.maxDate && DateHelper.isGreater(startDate, this._options.maxDate))) {
            startDate = this._options.minDate ? this._options.minDate : this._options.maxDate;
        }
        var /** @type {?} */ currentDisplayMonth = new Date(startDate);
        currentDisplayMonth.setDate(1);
        DateHelper.resetTime(currentDisplayMonth);
        this._options.currentDisplayMonth = currentDisplayMonth;
        this._options.startDatePlaceholder = this._options.startDatePlaceholder ? this._options.startDatePlaceholder : 'Select';
        this._options.endDatePlaceholder = this._options.endDatePlaceholder ? this._options.endDatePlaceholder : 'Select';
        this._options.startDateFormat = this._options.startDateFormat ? this._options.startDateFormat : 'mediumDate';
        this._options.endDateFormat = this._options.endDateFormat ? this._options.endDateFormat : 'mediumDate';
        this._options.range = this._options.range || false;
        this._options.showMonths = this._options.showMonths || 1;
        this._options.dayListKind = this._options.dayListKind || 'blacklist';
        this._options.enableTodayAction = this._options.enableTodayAction || false;
        this._options.enableClearAction = this._options.enableClearAction || false;
        this._options.useSelectorWidth = this._options.useSelectorWidth || false;
        // Force triggering of new selections to ensure that are still valid dates
        this.setStartDate(this._startDate);
        this.setEndDate(this._endDate);
    };
    /**
     * @param {?=} day
     * @return {?}
     */
    HacDatepickerComponent.prototype.setStartDate = function (day) {
        this.startDate = day;
        this.startDateChange.emit(this._startDate);
        this.hoverDate = DateHelper.ensureDateObject(this._startDate);
        this.pushNewModelChange();
    };
    /**
     * @param {?=} day
     * @return {?}
     */
    HacDatepickerComponent.prototype.setEndDate = function (day) {
        this.endDate = day;
        this.endDateChange.emit(this._endDate);
        this.hoverDate = DateHelper.ensureDateObject(this._endDate);
        this.pushNewModelChange();
    };
    /**
     * @return {?}
     */
    HacDatepickerComponent.prototype.getLastRangeDay = function () {
        if (this.getSelectionKind() === 'start') {
            return this._endDate;
        }
        return this.hoverDate || this._endDate;
    };
    /**
     * @param {?} day
     * @param {?=} selectionKind
     * @return {?}
     */
    HacDatepickerComponent.prototype.isInDayList = function (day, selectionKind) {
        var /** @type {?} */ dayList = selectionKind === 'start' ?
            this._options.dayListStartDate :
            this._options.dayListEndDate;
        if (!dayList)
            return false;
        var /** @type {?} */ year = dayList[day.day.getFullYear()];
        if (!year)
            return false;
        var /** @type {?} */ month = year[day.day.getMonth() + 1];
        if (!month)
            return false;
        return month.indexOf(day.day.getDate()) > -1;
    };
    /**
     * @return {?}
     */
    HacDatepickerComponent.prototype.animate = function () {
        var _this = this;
        var /** @type {?} */ animatingCssClass = 'hac-cal-wrapper--animating';
        this.elementRef.nativeElement.querySelector('.hac-cal-wrapper').classList.remove(animatingCssClass);
        setTimeout(function () { return _this.elementRef.nativeElement.querySelector('.hac-cal-wrapper').classList.add(animatingCssClass); }, 100);
    };
    /**
     * @return {?}
     */
    HacDatepickerComponent.prototype.getMinDate = function () {
        return this._options.minDate;
    };
    /**
     * @return {?}
     */
    HacDatepickerComponent.prototype.getMaxDate = function () {
        return this._options.maxDate;
    };
    /**
     * @return {?}
     */
    HacDatepickerComponent.prototype.emitFocusEvent = function () {
        var /** @type {?} */ event = document.createEvent('Event');
        var /** @type {?} */ focusKindEvent = this._collapsed ? 'customfocusout' : 'customfocus';
        event.initEvent(focusKindEvent, true, true);
        this.elementRef.nativeElement.querySelector('.js-focus').dispatchEvent(event);
    };
    HacDatepickerComponent.decorators = [
        { type: Component, args: [{
                    selector: 'hac-datepicker',
                    template: "<div class=\"hac-cal dropdown form-control\" [ngClass]=\"{ 'open show': !collapsed }\"> <!-- selector wrapper --> <div class=\"hac-cal-selectorwrapper\"> <input type=\"text\" style=\"display: none;\" *ngIf=\"options.elementId\" id=\"{{options.elementId}}\" class=\"js-focus\" (focus)=\"focus()\" (focusout)=\"focusout()\"> <i class=\"fa fa-calendar\" aria-hidden=\"true\" (click)=\"toggleCalendar($event)\"></i> <!-- start/single date --> <span class=\"hac-cal-date hac-cal-startdate\" (click)=\"forceSelectionKind('start')\" [ngClass]=\"{'hac-cal-date--is-active': getSelectionKind() === 'start' && !collapsed, 'hac-cal-date--is-disabled': disabled }\"> <span *ngIf=\"startDate\"> {{startDate | date: options?.startDateFormat}} </span> <span *ngIf=\"!startDate\">{{options?.startDatePlaceholder}}</span> </span> <!-- end date --> <span class=\"hac-cal-date hac-cal-enddate\" *ngIf=\"options?.range\" (click)=\"forceSelectionKind('end')\" [ngClass]=\"{'hac-cal-date--is-active': getSelectionKind() === 'end' && !collapsed, 'hac-cal-date--is-disabled': disabled }\"> <span *ngIf=\"endDate\"> {{endDate | date: options?.endDateFormat}} </span> <span *ngIf=\"!endDate\">{{options?.endDatePlaceholder}}</span> </span> <i class=\"fa fa-angle-down fa-2x\" aria-hidden=\"true\" (click)=\"toggleCalendar($event)\"></i> </div> <!-- calendar wrapper --> <ul class=\"hac-cal-wrapper dropdown-menu\"  [ngClass]=\"{ 'hac-cal-wrapper--is-opened': !collapsed }\" [hacCopyWidth]=\"getSelectorElem()\" widthCssProperty=\"width\" [skip]=\"collapsed || !options.useSelectorWidth\"> <li class=\"hac-cal-month\" *ngFor=\"let calendar of calendars; let i = index\"> <!-- calendar header --> <div class=\"hac-cal-header\"> <div class=\"hac-cal-arrow hac-cal-arrow--left\"> <i *ngIf=\"isPrevArrowVisible(i)\" (click)=\"prevMonth($event)\" class=\"hac-cal-chevron hac-cal-chevron--left\" aria-hidden=\"true\"></i> </div> <span class=\"hac-cal-monthlabel\"> <span class=\"hac-cal-monthlabel--year\">{{calendar.month | date : 'y '}}</span> <span class=\"hac-cal-monthlabel--month\">{{calendar.month | date : 'MMMM'}}</span> </span> <div class=\"hac-cal-arrow hac-cal-arrow--right\"> <i *ngIf=\"isNextArrowVisible(i)\" (click)=\"nextMonth($event)\" class=\"hac-cal-chevron hac-cal-chevron--right\" aria-hidden=\"true\"></i> </div> </div> <div class=\"hac-cal-weeks\"> <!-- week day list --> <div class=\"hac-cal-weedays\"> <div *ngFor=\"let weekDay of weekDayList\" class=\"hac-cal-weekday\"> {{ weekDay | hacWeekDayFormatter }} </div> </div> <!-- weeks --> <div *ngFor=\"let week of calendar.weeks\" class=\"hac-cal-week\"> <div *ngFor=\"let day of week.days\"  class=\"hac-cal-day\" (click)=\"select(day)\" (mouseenter)=\"setHoverDate(day)\" (mouseleave)=\"setHoverDate(null)\" [ngClass]=\"{  'hac-cal-day--is-selected': isSelected(day), 'hac-cal-day--is-hidden': !calendar.contains(day), 'hac-cal-day--is-disabled': isDisabled(day), 'hac-cal-day--is-today': day.isToday(), 'hac-cal-day--is-start': isStart(day), 'hac-cal-day--is-end': isEnd(day), 'hac-cal-day--in-range': inRange(day), 'hac-cal-day--in-rangehover': inRange(day, true) }\"> <div class=\"hac-cal-daynum\"  title=\"{{ day?.day | date : 'fullDate'}}\"> {{ day?.day | date : 'd'}} </div> </div> </div> </div> <div class=\"btn-group\" *ngIf=\"i === 0\"> <button *ngIf=\"isTodayActionEnabled()\" class=\"btn btn-default\" type=\"button\" (click)=\"selectToday()\"> <i class=\"fa fa-calendar\"></i> </button> <button *ngIf=\"isClearActionEnabled()\" class=\"btn btn-danger\" type=\"button\" (click)=\"clearDates()\"> <i class=\"fa fa-times\"></i> </button> </div> </li> </ul> </div>",
                    providers: [
                        DatePipe,
                        HacWeekDayFormatter,
                        {
                            provide: NG_VALUE_ACCESSOR,
                            useExisting: forwardRef(function () { return HacDatepickerComponent; }),
                            multi: true
                        },
                        {
                            provide: NG_VALIDATORS,
                            useExisting: forwardRef(function () { return HacDatepickerComponent; }),
                            multi: true,
                        }
                    ]
                },] },
    ];
    /**
     * @nocollapse
     */
    HacDatepickerComponent.ctorParameters = function () { return [
        { type: ElementRef, },
    ]; };
    HacDatepickerComponent.propDecorators = {
        'startDate': [{ type: Input },],
        'endDate': [{ type: Input },],
        'startDateChange': [{ type: Output },],
        'endDateChange': [{ type: Output },],
        'selected': [{ type: Input },],
        'selectedChange': [{ type: Output },],
        'options': [{ type: Input },],
        'disabled': [{ type: Input },],
        'onClick': [{ type: HostListener, args: ['document:click', ['$event'],] },],
    };
    return HacDatepickerComponent;
}());

var HacCopyWidthDirective = (function () {
    /**
     * @param {?} elementRef
     */
    function HacCopyWidthDirective(elementRef) {
        this.elementRef = elementRef;
        this.widthCssProperty = 'width';
        this.skip = false;
    }
    /**
     * @return {?}
     */
    HacCopyWidthDirective.prototype.ngOnInit = function () {
        this.calcWidth();
    };
    /**
     * @return {?}
     */
    HacCopyWidthDirective.prototype.ngDoCheck = function () {
        this.calcWidth();
    };
    /**
     * @param {?} event
     * @return {?}
     */
    HacCopyWidthDirective.prototype.onResize = function (event) {
        this.calcWidth();
    };
    /**
     * @return {?}
     */
    HacCopyWidthDirective.prototype.calcWidth = function () {
        if (this.skip) {
            this.setWidth(null);
            return;
        }
        if (!this.hacCopyWidth) {
            this.setWidth(0);
            return;
        }
        var /** @type {?} */ computedStyle = getComputedStyle(this.hacCopyWidth, null);
        var /** @type {?} */ borderLeft = parseFloat(computedStyle.borderLeftWidth.replace('px', ''));
        var /** @type {?} */ borderRight = parseFloat(computedStyle.borderRightWidth.replace('px', ''));
        var /** @type {?} */ finalWidth = this.hacCopyWidth.offsetWidth - borderLeft - borderRight;
        this.setWidth(finalWidth);
    };
    /**
     * @param {?} width
     * @return {?}
     */
    HacCopyWidthDirective.prototype.setWidth = function (width) {
        this.elementRef.nativeElement.style[this.widthCssProperty] = width != null ? width + "px" : null;
    };
    HacCopyWidthDirective.decorators = [
        { type: Directive, args: [{
                    selector: '[hacCopyWidth]'
                },] },
    ];
    /**
     * @nocollapse
     */
    HacCopyWidthDirective.ctorParameters = function () { return [
        { type: ElementRef, },
    ]; };
    HacCopyWidthDirective.propDecorators = {
        'hacCopyWidth': [{ type: Input },],
        'widthCssProperty': [{ type: Input },],
        'skip': [{ type: Input },],
        'onResize': [{ type: HostListener, args: ['window:resize', ['$event'],] },],
    };
    return HacCopyWidthDirective;
}());

var HacLabelFocusDirective = (function () {
    /**
     * @param {?} elementRef
     */
    function HacLabelFocusDirective(elementRef) {
        this.elementRef = elementRef;
        this.hacLabelFocus = 'focus';
        this.setFocused = this.setFocused.bind(this);
        this.setBlur = this.setBlur.bind(this);
    }
    /**
     * @return {?}
     */
    HacLabelFocusDirective.prototype.ngDoCheck = function () {
        var /** @type {?} */ targetElem = document.querySelector("#" + this.elementRef.nativeElement.htmlFor);
        if (targetElem === this.targetElem)
            return;
        if (targetElem) {
            targetElem.addEventListener('focus', this.setFocused, true);
            targetElem.addEventListener('focusout', this.setBlur, true);
            targetElem.addEventListener('customfocus', this.setFocused, true);
            targetElem.addEventListener('customfocusout', this.setBlur, true);
        }
        this.removeCurrentEventListeners();
        this.targetElem = targetElem;
    };
    /**
     * @return {?}
     */
    HacLabelFocusDirective.prototype.ngOnDestroy = function () {
        this.removeCurrentEventListeners();
    };
    /**
     * @param {?} e
     * @return {?}
     */
    HacLabelFocusDirective.prototype.setFocused = function (e) {
        this.focus(true);
    };
    /**
     * @param {?} e
     * @return {?}
     */
    HacLabelFocusDirective.prototype.setBlur = function (e) {
        this.focus(false);
    };
    /**
     * @param {?} isFocused
     * @return {?}
     */
    HacLabelFocusDirective.prototype.focus = function (isFocused) {
        isFocused ?
            this.elementRef.nativeElement.classList.add(this.hacLabelFocus) :
            this.elementRef.nativeElement.classList.remove(this.hacLabelFocus);
    };
    /**
     * @return {?}
     */
    HacLabelFocusDirective.prototype.removeCurrentEventListeners = function () {
        if (this.targetElem) {
            this.targetElem.removeEventListener('focus', this.setFocused);
            this.targetElem.removeEventListener('focusout', this.setBlur);
            this.targetElem.removeEventListener('customfocus', this.setFocused);
            this.targetElem.removeEventListener('customfocusout', this.setBlur);
        }
    };
    HacLabelFocusDirective.decorators = [
        { type: Directive, args: [{
                    selector: '[hacLabelFocus]'
                },] },
    ];
    /**
     * @nocollapse
     */
    HacLabelFocusDirective.ctorParameters = function () { return [
        { type: ElementRef, },
    ]; };
    HacLabelFocusDirective.propDecorators = {
        'hacLabelFocus': [{ type: Input },],
    };
    return HacLabelFocusDirective;
}());

var HacModule = (function () {
    function HacModule() {
    }
    /**
     * @return {?}
     */
    HacModule.forRoot = function () {
        return {
            ngModule: HacModule
        };
    };
    HacModule.decorators = [
        { type: NgModule, args: [{
                    declarations: [
                        HacDropdownComponent,
                        HacDropdownFilterPipe,
                        HacDatepickerComponent,
                        HacWeekDayFormatter,
                        HacCopyWidthDirective,
                        HacLabelFocusDirective
                    ],
                    imports: [
                        BrowserModule,
                        FormsModule,
                        CommonModule
                    ],
                    exports: [
                        HacDropdownComponent,
                        HacDatepickerComponent,
                        HacCopyWidthDirective,
                        HacLabelFocusDirective
                    ]
                },] },
    ];
    /**
     * @nocollapse
     */
    HacModule.ctorParameters = function () { return []; };
    return HacModule;
}());

export { HacDatepickerComponent, HacWeekDayFormatter, HacDropdownComponent, HacDropdownFilterPipe, HacCopyWidthDirective, HacLabelFocusDirective, HacModule };
